#********************************************************************************************************
# Name: Escort for Battlefield 2142
# Author: Mach10 <mach10@comcast.net>
# Version 0.1 
# Copyright  MNM Creations, LLC  - www.bfeditor.org/mach10
#----------------------------------------------------------------------------------------------------------------------------
# Conditions and Limitations: You may use this file locally, on listening and dedicated servers. You MAY NOT 
# copy, redistribute or post this file with out the expressed permission from the author/copyright holder.
# You may contact me and request these rights via e-mail: mach10@comcast.net
#This software is provided "as is". In no event shall the author, authors or contributors be held
#liable for  any direct, indirect, incidental, special, exemplary, or consequential damages (including)
#but not limited to, procurement of substitute good or services; loss of use, data or profits; or 
#business interruption) however caused and on any theory of liability, whether in contract, strict liability
#arising in any way out of the use of this software, even if advised of the possibility of such damage.
#=====
# Purpose: The file creates a new game mode for Battlefield 2142, Escort.
#=====
#*********************************************************************************************************

TAKEOVERTYPE_CAPTURE = 1
TAKEOVERTYPE_NEUTRALIZE = 2

SCORE_CAPTURE = 2
SCORE_NEUTRALIZE = 1
SCORE_CAPTUREASSIST = 1
SCORE_NEUTRALIZEASSIST = 1
SCORE_DEFEND = 1

SCORE_DEFEND_VIP = 2
SCORE_KILL_VIP = 10
SCORE_TK_VIP = -15
SCORE_ASSIST_EXIT = 1

Top = 0
Middle = 1
Bottom = 2

class Escort_Data:
	def __init__(self):
		self.VIP = None
		self.noVIPTimer = None
		self.vipAttackers = []
		self.vipTriggerId = None
		self.displayWarning = None
		self.vipInSafeZone = False
		self.VIPDead = False
		self.VIPWasTkd = False
		self.attackersUpdater = None


Escort = Escort_Data()


import host
import bf2
import math
from game.scoringCommon import addScore, RPL
from bf2 import g_debug

g_controlPoints = [] # cache, as this map won't change



def init():
	# events hook
	host.registerGameStatusHandler(onGameStatusChanged)
	if host.sgl_getIsAIGame() == 1:
		host.sh_setEnableCommander(1)
	else:
		host.sh_setEnableCommander(1)
		
	host.registerHandler('TimeLimitReached', onTimeLimitReachedEscort, 1)	

	if g_debug: print "gpm_escort.py initialized"
		
		
		
def deinit():
	bf2.triggerManager.destroyAllTriggers()
	global g_controlPoints
	g_controlPoints = []
	host.unregisterGameStatusHandler(onGameStatusChanged)
	
	host.unregisterHandler('TicketLimitReached', onTicketLimitReachedEscort)
	host.unregisterHandler('PlayerDeath', onPlayerDeathEscort)
	host.unregisterHandler('PlayerKilled', onPlayerKilledEscort)
	host.unregisterHandler('PlayerRevived', onPlayerRevivedEscort)
	host.unregisterHandler('PlayerSpawn', onPlayerSpawnEscort)
	host.unregisterHandler('EnterVehicle', onEnterVehicleEscort)
	host.unregisterHandler('ExitVehicle', onExitVehicleEscort)
	host.unregisterHandler('PlayerDisconnect', onPlayerDisconnectEscort)
	host.unregisterHandler('ChangedCommander', onChangedCommanderEscort)
	
	if g_debug: print "gpm_escort.py uninitialized"



def onGameStatusChanged(status):
	global g_controlPoints
	clearOldTimers()
	Escort.VIPWasTkd = False
	Escort.VIPDead = False
	Escort.vipInSafeZone = False
	
	if status == bf2.GameStatus.Playing: 

		# add control point triggers
		g_controlPoints = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint')
		for obj in g_controlPoints:
			if int(obj.cp_getParam('unableToChangeTeam')) == 1 and int(obj.cp_getParam('team')) == 2:	
				radius = float(obj.getTemplateProperty('radius'))
				isHemi = int(obj.cp_getParam('isHemisphere'))
				if isHemi != 0:
					id = bf2.triggerManager.createHemiSphericalTrigger(obj, onDestinationTrigger, '<<PCO>>', radius, (1, 2, 3))
				else:
					id = bf2.triggerManager.createRadiusTrigger(obj, onDestinationTrigger, '<<PCO>>', radius, (1, 2, 3))			
				obj.triggerId = id
				
			else:
				radius = float(obj.getTemplateProperty('radius'))
				isHemi = int(obj.cp_getParam('isHemisphere'))
				if isHemi != 0:
					id = bf2.triggerManager.createHemiSphericalTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))
				else:
					id = bf2.triggerManager.createRadiusTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))			
				obj.triggerId = id
				obj.lastAttackingTeam = 0
				if obj.cp_getParam('team') > 0:
					obj.flagPosition = Top
				else:
					obj.flagPosition = Bottom

		host.registerHandler('ControlPointChangedOwner', onCPStatusChangeEscort)

		# setup ticket system
		ticketsTeam1 = calcStartTickets(bf2.gameLogic.getDefaultTickets(1))
		ticketsTeam2 = calcStartTickets(bf2.gameLogic.getDefaultTickets(2))
		
		bf2.gameLogic.setTickets(1, ticketsTeam1)
		bf2.gameLogic.setTickets(2, ticketsTeam2)

		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)

		bf2.gameLogic.setTicketLimit(1, 1, 0)
		bf2.gameLogic.setTicketLimit(2, 1, 0)
		bf2.gameLogic.setTicketLimit(1, 2, 10)
		bf2.gameLogic.setTicketLimit(2, 2, 10)
		bf2.gameLogic.setTicketLimit(1, 3, int(ticketsTeam1*0.1))
		bf2.gameLogic.setTicketLimit(2, 3, int(ticketsTeam2*0.1))
		bf2.gameLogic.setTicketLimit(1, 4, int(ticketsTeam1*0.2))
		bf2.gameLogic.setTicketLimit(2, 4, int(ticketsTeam1*0.2))
		
		host.registerHandler('TicketLimitReached', onTicketLimitReachedEscort)

		# player events
		host.registerHandler('PlayerDeath', onPlayerDeathEscort)
		host.registerHandler('PlayerKilled', onPlayerKilledEscort)
		host.registerHandler('PlayerRevived', onPlayerRevivedEscort)
		host.registerHandler('PlayerSpawn', onPlayerSpawnEscort)
		host.registerHandler('EnterVehicle', onEnterVehicleEscort)
		host.registerHandler('ExitVehicle', onExitVehicleEscort)
		host.registerHandler('PlayerDisconnect', onPlayerDisconnectEscort)
		host.registerHandler('ChangedCommander', onChangedCommanderEscort) 


		if g_debug: print "Escort gamemode initialized."
		
			
		if not Escort.VIP:
			host.rcon_invoke("game.sayAll \"" + "PAC Team must elect a commander (VIP) or forfeit!" + "\"")
			Escort.noVIPTimer = bf2.Timer(noVIP, 60, 1)

	else:
		clearOldTimers()
		bf2.triggerManager.destroyAllTriggers()
		g_controlPoints = []
	
def clearOldTimers():
	try:
		Escort.noVIPTimer.destroy()
		Escort.noVIPTimer = None
	except: pass
	
	try:
		Escort.attackersUpdater.destroy()
		Escort.attackersUpdater = None
	except: pass
	
	Escort.VIP = None
	Escort.vipAttackers = []
	Escort.vipTriggerId = None
	Escort.displayWarning = None
	Escort.vipInSafeZone = False
	Escort.VIPDead = False
	Escort.VIPWasTkd = False
	
def calcStartTickets(mapDefaultTickets):
	return int(mapDefaultTickets * (bf2.serverSettings.getTicketRatio() / 100.0))
	
def onPlayerDisconnectEscort(player):	
	if player.index == Escort.VIP.index:
		Escort.VIP = None
		host.rcon_invoke("game.sayAll \"" + "PAC Team has 60 seconds to elect a commander (VIP) or forfeit!" + "\"")
		Escort.noVIPTimer = bf2.Timer(noVIP, 60, 1)

			
def onChangedCommanderEscort(team, oldCommander, newCommander):
	if not newCommander:
		if team == 2:
			Escort.VIP = None
			host.rcon_invoke("game.sayAll \"" + "PAC Team has 60 seconds to elect a commander (VIP) or forfeit!" + "\"")
			Escort.noVIPTimer = bf2.Timer(noVIP, 60, 1)
			try:
				Escort.attackersUpdater.destroy()
				Escort.attackersUpdater = None
			except: pass
	
	else:	
		if team == 2:
			try:
				Escort.noVIPTimer.destroy()
				Escort.noVIPTimer = None
			except: pass
		
			Escort.VIPDead = False
			Escort.VIPWasTkd = False
			Escort.VIP = newCommander
			vehicle = newCommander.getDefaultVehicle()
			Escort.vipTriggerId = bf2.triggerManager.createRadiusTrigger(vehicle, onEscortTrigger, '<<PCO>>', 5, 1)
			Escort.attackersUpdater = bf2.Timer(clearOldAttackers, 0, 1)
			Escort.attackersUpdater.setRecurring(2.0)
		
# actual ticket loss calculation function
def calcTicketLossForTeam(team, otherTeamAreaValue, otherTeamAreaOverweight):
	if otherTeamAreaValue >= 100 and otherTeamAreaOverweight > 0:
		ticketLossPerSecond = (bf2.gameLogic.getDefaultTicketLossPerMin(team) / 60.0) * (otherTeamAreaOverweight / 100.0)
		return ticketLossPerSecond
	else:
		return 0

def onTimeLimitReachedEscort(value):
	winner = 1
	victoryType = 3

	host.sgl_endGame(winner, victoryType, 0)

	
DOWNWARDS = -1
UPWARDS = 1	

# called when tickets reach a predetermined limit (negativ value means that the tickets have become less than the limit)
def onTicketLimitReachedEscort(team, limitId):
	if (limitId == -1):
		if (team == 1):
			winner = 2
		
		elif (team == 2):
			winner = 1
		
		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)
	
		host.sgl_endGame(winner, 3, 0)
		
# update ticket system
def updateTicketLoss():
	losingTeam = -1
	winningTeam = -1
	#if the VIP is dead, end of round
	if Escort.VIPDead:
		if not Escort.VIPWasTkd:
			losingTeam = 2
			winningTeam = 1
			host.sgl_endGame(winningTeam, 3, 0)
			return
		
		else:
			losingTeam = 1
			winningTeam = 2
			host.sgl_endGame(winningTeam, 3, 0)
			return
			
	
	#if the VIP made it to safe zone, end of round
	if Escort.vipInSafeZone:
		losingTeam = 1
		winningTeam = 2
		host.sgl_endGame(winningTeam, 3, 0)
		return
	
	# check if there is anyone alive
	foundLivingPlayer = False
	for p in bf2.playerManager.getPlayers():
		if p.getTeam() == losingTeam and p.isAlive() and not p.isAutoController():
			foundLivingPlayer = True
			break
				
	if not foundLivingPlayer:
		# drop tickets
		ticketLossPerSecond = bf2.gameLogic.getDefaultTicketLossAtEndPerMin()
		bf2.gameLogic.setTicketChangePerSecond(losingTeam, -ticketLossPerSecond)
		bf2.gameLogic.setTicketChangePerSecond(winningTeam, 0)
			
		return
	
	
# called when someone enters or exits cp radius
def onCPTrigger(triggerId, cp, vehicle, enter, userData):
	if not cp.isValid(): return

	if vehicle and vehicle.getParent(): return
	
	# can this cp be captured at all?
	if cp.cp_getParam('unableToChangeTeam') != 0:
		return					
		
	playersInVehicle = None	
	if vehicle:
		playersInVehicle = vehicle.getOccupyingPlayers()
	
	if enter:
		for p in playersInVehicle:
			cp = getOccupyingCP(p)
			if cp != None:
				if not p.getIsInsideCP():
					if g_debug: print "Resetting enterPctAt for player ", p.getName()
					p.enterCpAt = host.timer_getWallTime()
	
	if vehicle:	
		for p in playersInVehicle:
			# only count first player in a vehicle
			if p == playersInVehicle[0]:  
				if g_debug: print p.index, " is in radius. v=", vehicle.templateName
				p.setIsInsideCP(enter)	
			else:
				p.setIsInsideCP(0)
				if enter == 1:
					bf2.gameLogic.sendHudEvent(p, 71, 49) #71 = HEEnableHelpMessage, 49 = VHMExitToCaptureFlag;


	# count people in radius
	team1Occupants = 0
	team2Occupants = 0

	pcos = bf2.triggerManager.getObjects(cp.triggerId)
	for o in pcos:
		if not o: continue # you can get None in the result tuple when the host can't figure out what object left the trigger
		if o.getParent(): continue # getOccupyingPlayers returns all players downwards in the hierarchy, so dont count them twice
		occupyingPlayers = o.getOccupyingPlayers()
		for p in occupyingPlayers:
			
			# only count first player in a vehicle
			if p != occupyingPlayers[0]: 
				continue
				
			if p.isAlive() and not p.isManDown():
				
				if not p.killed:
					if g_debug: print "Player ", p.getName(), " is inside radius"

					if p.getTeam() == 1:
						team1Occupants += 1
					elif p.getTeam() == 2:
						team2Occupants += 1


	# determine who is taking control
	team1OverWeight = team1Occupants - team2Occupants
	attackOverWeight = 0

	if team1OverWeight > 0:
		attackingTeam = 1
	elif team1OverWeight < 0:
		attackingTeam = 2
	else:
		attackingTeam = 0

	
	if team1Occupants == 0 and team2Occupants == 0:

		# nobody here, slowly go back to owning team
		if cp.cp_getParam('team') == 0:
			attackOverWeight = -0.5
		else:
			attackOverWeight = 0.5
			
		timeToChangeControl = cp.cp_getParam('timeToLoseControl')

	else:

		# raise flag if already ours, or at bottom and neutral. Otherwise lower first.
		if cp.cp_getParam('flag') == attackingTeam or (cp.flagPosition == Bottom and cp.cp_getParam('team') == 0):

			# our flag, raise
			attackOverWeight = abs(team1OverWeight)
			timeToChangeControl = cp.cp_getParam('timeToGetControl')
		else:

			# other team raised flag, lower first
			attackOverWeight = - abs(team1OverWeight)
			timeToChangeControl = cp.cp_getParam('timeToLoseControl')


	if cp.cp_getParam('onlyTakeableByTeam') != 0 and cp.cp_getParam('onlyTakeableByTeam') != attackingTeam:
		return


	# flag can only be changed when at bottom
	if cp.flagPosition == Bottom:
		cp.cp_setParam('flag', attackingTeam)


	# calculate flag raising/lowering speed
	if timeToChangeControl > 0:
		takeOverChangePerSecond = 1.0 * attackOverWeight / timeToChangeControl
	else:
		takeOverChangePerSecond = 0.0

	if (cp.flagPosition == Top and takeOverChangePerSecond > 0) or (cp.flagPosition == Bottom and takeOverChangePerSecond < 0):
		takeOverChangePerSecond = 0.0

	if abs(takeOverChangePerSecond) > 0:
		cp.flagPosition = Middle
				
	cp.cp_setParam('takeOverChangePerSecond', takeOverChangePerSecond)
	

def onDestinationTrigger(triggerId, cp, vehicle, enter, userData):
	if not cp.isValid(): return

	if vehicle and vehicle.getParent(): return				
	
	if enter:
		for p in vehicle.getOccupyingPlayers():
			if p.isAlive() and not p.isManDown() and not p.killed:
				if p.getVehicle() == p.getDefaultVehicle(): #only check for players on foot
					if p.getTeam() == 1 and p.isCommander(): #only check for the VIP
						host.rcon_invoke("game.sayAll \"" + "VIP has reached the destination safely" + "\"")
						Escort.vipInSafeZone = True
						
	if Escort.vipInSafeZone:
		# count people in radius
		team1Occupants = []

		pcos = bf2.triggerManager.getObjects(cp.triggerId)
		for o in pcos:
			if not o: continue # you can get None in the result tuple when the host can't figure out what object left the trigger
			if o.getParent(): continue # getOccupyingPlayers returns all players downwards in the hierarchy, so dont count them twice
			occupyingPlayers = o.getOccupyingPlayers()
			for p in occupyingPlayers:
				if p.isAlive() and not p.isManDown() and not p.killed:
						if p.getTeam() == 1:
							if len(team1Occupants) == 0:
								team1Occupants = [p]
							else:
								team1Occupants += [p]
						
		# deal score
		for p in team1Occupants:
			p.score.cpCaptures += 1
			addScore(p, SCORE_ASSIST_EXIT, RPL)
			
		updateTicketLoss()


def onEscortTrigger(triggerId, player, vehicle, enter, userData):
	return
	
def clearOldAttackers(data):
	Escort.vipAttackers = []
	pcos = bf2.triggerManager.getObjects(Escort.vipTriggerId)
	for obj in pcos:
		if not obj: continue
		occupyingPlayers = obj.getOccupyingPlayers()
		for p in occupyingPlayers:	
			if p.getTeam() == 1:	
				Escort.vipAttackers.append(p)

def isAttacker(player, team):
	found = 0
	if team == 1:
		for storedAttacker in Escort.vipAttackers:
			if storedAttacker.getName() == player.getName():	
				found = 1
		return found
	

# called when a control point flag reached top or bottom
def onCPStatusChangeEscort(cp, top):
	playerId = -1
	takeoverType = -1
	newTeam = -1
	scoringTeam = -1
	
	if top:	cp.flagPosition = Top
	else:   cp.flagPosition = Bottom
	
	# determine capture / neutralize / defend
	if cp.cp_getParam('team') != 0:

		if top:
			# regained flag, do nothing
			pass
			
		else:
			# neutralize
			newTeam = 0
			if cp.cp_getParam('team') == 1:
				scoringTeam = 2
			else:
				scoringTeam = 1
				
			takeoverType = TAKEOVERTYPE_NEUTRALIZE

	else:

		if top:
			# capture
			newTeam = cp.cp_getParam('flag')
			scoringTeam = newTeam
			takeoverType = TAKEOVERTYPE_CAPTURE

		else:
			# hit bottom, but still neutral
			pass

	
	# scoring
	if takeoverType > 0:
		pcos = bf2.triggerManager.getObjects(cp.triggerId)
	
		# count number of players
		scoringPlayers = []
		firstPlayers = []
		for o in pcos:
			if o.getParent(): continue

			occupyingPlayers = o.getOccupyingPlayers()
			for p in occupyingPlayers:
			
				# only count first player in a vehicle
				if p != occupyingPlayers[0]: 
					continue
					
				if p.isAlive() and not p.isManDown() and p.getTeam() == scoringTeam:
					if len(firstPlayers) == 0 or p.enterCpAt < firstPlayers[0].enterCpAt:
						firstPlayers = [p]
					elif p.enterCpAt == firstPlayers[0].enterCpAt:
						firstPlayers += [p]
					
					if not p in scoringPlayers:
						scoringPlayers += [p]
	
		# deal score
		for p in scoringPlayers:
			oldScore = p.score.score;
			if takeoverType == TAKEOVERTYPE_CAPTURE:
				if p in firstPlayers:
					p.score.cpCaptures += 1
					addScore(p, SCORE_CAPTURE, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 0) #12 = Conquest, 0 = Capture
					playerId = p.index
				else:
					p.score.cpAssists += 1
					addScore(p, SCORE_CAPTUREASSIST, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 2) #12 = Conquest, 2 = Assist


			elif takeoverType == TAKEOVERTYPE_NEUTRALIZE:
				if p in firstPlayers:
					p.score.cpNeutralizes += 1
					addScore(p, SCORE_NEUTRALIZE, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 3) #12 = Conquest, 3 = Neutralize
					playerId = p.index
				else:
					p.score.cpNeutralizeAssists += 1
					addScore(p, SCORE_NEUTRALIZEASSIST, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 4) #12 = Conquest, 4 = Neutralize assist
					
	
	
	# update control point	
	cp.cp_setParam('playerId', playerId) #always set player first
	if newTeam != -1 and cp.cp_getParam('team') != newTeam:
		cp.cp_setParam('team', newTeam)
	onCPTrigger(cp.triggerId, cp, 0, 0, 0)
	updateTicketLoss()


def onPlayerDeathEscort(victim, vehicle):		
	# punish team with one ticket
	if victim == None:
		return
	team = victim.getTeam()
	teamTickets = bf2.gameLogic.getTickets(team)
	teamTickets -= 1
	bf2.gameLogic.setTickets(team, teamTickets)	

	#check if it was the commander
	if team == 1 and victim.isCommander():
		Escort.VIPDead = True
		victim.setTimeToSpawn(60)
		updateTicketLoss()
		
	
	# check if it was the last player
	foundLivingPlayer = False
	for p in bf2.playerManager.getPlayers():
		if p != victim and p.getTeam() == victim.getTeam() and p.isAlive() and not p.isAutoController():
			foundLivingPlayer = True
	
	if not foundLivingPlayer:
		updateTicketLoss()
		
	
			
def onPlayerKilledEscort(victim, attacker, weapon, assists, object):
	if not victim: 
		return
	
	victim.killed = True
	
	# update flag takeover status if victim was in a CP radius
	cp = getOccupyingCP(victim)
	if cp != None:
		onCPTrigger(-1, cp, victim.getVehicle(), False, None)

		# give defend score if killing enemy within cp radius
		if attacker != None and attacker.getTeam() != victim.getTeam()\
		   and cp.cp_getParam('unableToChangeTeam') == 0 and cp.cp_getParam('onlyTakeableByTeam') == 0:
		
			if cp != None and cp.cp_getParam('team') == attacker.getTeam():
				attacker.score.cpDefends += 1
				addScore(attacker, SCORE_DEFEND, RPL)
				bf2.gameLogic.sendGameEvent(attacker, 12, 1) #12 = Conquest, 1 = Defend
				
	if isAttacker(victim, victim.getTeam()):
		if victim.getTeam() == attacker.getTeam():
			pass # he tk'd someone, give him nothing
		else:
			host.rcon_invoke("game.sayAll \"" + attacker.getName() + " has defended the VIP from" + victim.getName() + "\"")
			attacker.score.cpDefends += 1
			addScore(attacker, SCORE_DEFEND_VIP, RPL)
	
	if Escort.VIP != None and victim.index == Escort.VIP.index:		
		if attacker.getTeam() == Escort.VIP.getTeam(): #friendly killed VIP
			Escort.VIPWasTkd = True
			attacker.score.TKs += 1
			addScore(attacker, SCORE_TK_VIP, RPL)
		
		elif attacker.getTeam() != Escort.VIP.getTeam(): #enemy killed VIP
			Escort.VIPWasTkd = False
			attacker.score.kills += 1
			addScore(attacker, SCORE_KILL_VIP, RPL)
			

def onPlayerRevivedEscort(victim, attacker):
	# update flag takeover status if victim was in a CP radius
	victim.killed = False
	
	if victim.index == Escort.VIP.index:
		Escort.VIP.wasTkd = False
	
	cp = getOccupyingCP(victim)
	if cp != None:
		onCPTrigger(-1, cp, victim.getVehicle(), True, None)
	
def onPlayerSpawnEscort(player, soldier):
	player.killed = False
	
	if not Escort.displayWarning:
		if not Escort.VIP:
			host.rcon_invoke("game.sayAll \"" + "PAC team has 120 seconds to elect a commander or forfeit!" + "\"")
			Escort.noVIPTimer = bf2.Timer(noVIP, 30, 1)
			
		Escort.displayWarning = True

	
def onEnterVehicleEscort(player, vehicle, freeSoldier = False):
	rootVehicle = bf2.objectManager.getRootParent(vehicle)
	# update flag takeover status if player in a CP radius
	if player == None:
		return
	
	cp = getOccupyingCP(player)
	playersInVehicle = rootVehicle.getOccupyingPlayers()
	if cp != None and player == playersInVehicle[0]:
		occupyingPlayers = rootVehicle.getOccupyingPlayers()
		for p in occupyingPlayers:
			p.setIsInsideCP(False)
		if cp.cp_getParam('unableToChangeTeam') == 0:
			player.setIsInsideCP(True)
	else:		
		player.setIsInsideCP(False)

def onExitVehicleEscort(player, vehicle):
	# Player remaining in vehicle might capture CP
	playersInVehicle = vehicle.getOccupyingPlayers()
	if len(playersInVehicle) != 0:
		remainingPlayer = playersInVehicle[0]
		if remainingPlayer != None:
			cpRm = getOccupyingCP(remainingPlayer)
			if cpRm != None and cpRm.cp_getParam('unableToChangeTeam') == 0:
				remainingPlayer.setIsInsideCP(True)

	# update flag takeover status if player in a CP radius
	cp = getOccupyingCP(player)
	if cp != None and cp.cp_getParam('unableToChangeTeam') == 0:
		player.setIsInsideCP(True)
	else:		
		player.setIsInsideCP(False)
		
# find cp that player is occupying, if any		
def getOccupyingCP(player):	
	if player == None: return

	vehicle = player.getVehicle()
	playerPos = vehicle.getPosition()
	
	# find closest CP
	closestCP = None
	if len(g_controlPoints) == 0: return None
	for obj in g_controlPoints:
		distanceTo = getVectorDistance(playerPos, obj.getPosition())
		if closestCP == None or distanceTo < closestCPdist:
			closestCP = obj
			closestCPdist = distanceTo
	
	# is the player in radius?
	pcos = bf2.triggerManager.getObjects(closestCP.triggerId)
	for o in pcos:
		if o == player.getDefaultVehicle():
			# Player is DEFAULT vehicle - this is needed when called from onEnterVehicle
			return closestCP
		else:
			for p in o.getOccupyingPlayers():
				if p == player:
					return closestCP
	return None

# get distance between two positions
def getVectorDistance(pos1, pos2):
	diffVec = [0.0, 0.0, 0.0]
	diffVec[0] = math.fabs(pos1[0] - pos2[0])
	diffVec[1] = math.fabs(pos1[1] - pos2[1])
	diffVec[2] = math.fabs(pos1[2] - pos2[2])
	 
	return math.sqrt(diffVec[0] * diffVec[0] + diffVec[1] * diffVec[1] + diffVec[2] * diffVec[2])

def noVIP(data):
	foundVIP = False
	for p in bf2.playerManager.getPlayers():
		if p.getTeam() == 1 and p.isCommander() and p.isAlive():
			foundVIP = True
			
	if not foundVIP:
		host.rcon_invoke("game.sayAll \"" + "PAC Team has no commander (VIP) and forfeits!" + "\"")
		Escort.VIP = None
		Escort.VIPWasTkd = False
		Escort.VIPDead = True
		updateTicketLoss()
	
