# flashpoint

TAKEOVERTYPE_CAPTURE = 1
TAKEOVERTYPE_NEUTRALIZE = 2

SCORE_CAPTURE = 2
SCORE_NEUTRALIZE = 1
SCORE_CAPTUREASSIST = 1
SCORE_NEUTRALIZEASSIST = 1
SCORE_DEFEND = 1

class Flashpoint_Data:
	def __init__(self):
		self.timer = None
		self.nextPointTimer = None
		self.nextPointTime = 5
		self.initialized = False
		self.previousTeam = -1
		self.currentTeam = -1
		


FP = Flashpoint_Data()


Top = 0
Middle = 1
Bottom = 2

import host
import bf2
import math
import random
from game.scoringCommon import addScore, RPL
from bf2 import g_debug

g_controlPoints = [] # cache, as this map won't change


def init():
	# events hook
	host.registerGameStatusHandler(onGameStatusChanged)
	if host.sgl_getIsAIGame() == 1:
		host.sh_setEnableCommander(1)
	else:
		host.sh_setEnableCommander(1)
		
	host.registerHandler('TimeLimitReached', onTimeLimitReachedFP, 1)	

	if g_debug: print "gpm_fp.py initialized"
		
		
		
def deinit():
	bf2.triggerManager.destroyAllTriggers()
	global g_controlPoints
	g_controlPoints = []
	
	host.unregisterGameStatusHandler(onGameStatusChanged)
	
	host.unregisterHandler('TimeLimitReached', onTimeLimitReachedFP)	
	host.unregisterHandler('ControlPointChangedOwner', onCPStatusChangeFP)
	host.unregisterHandler('TicketLimitReached', onTicketLimitReachedFP)
	host.unregisterHandler('PlayerDeath', onPlayerDeathFP)
	host.unregisterHandler('PlayerKilled', onPlayerKilledFP)
	host.unregisterHandler('PlayerRevived', onPlayerRevivedFP)
	host.unregisterHandler('PlayerSpawn', onPlayerSpawnFP)
	host.unregisterHandler('EnterVehicle', onEnterVehicleFP)
	host.unregisterHandler('ExitVehicle', onExitVehicleFP)
	
	if g_debug: print "gpm_fp.py uninitialized"



def onGameStatusChanged(status):
	global g_controlPoints
	initFlashPoint()
	if status == bf2.GameStatus.Playing: 

		# add control point triggers
		g_controlPoints = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint')
		for obj in g_controlPoints:
			radius = float(obj.getTemplateProperty('radius'))
			isHemi = int(obj.cp_getParam('isHemisphere'))
			if isHemi != 0:
				id = bf2.triggerManager.createHemiSphericalTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))
			else:
				id = bf2.triggerManager.createRadiusTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))			
			obj.triggerId = id
			obj.lastAttackingTeam = 0
			if obj.cp_getParam('team') > 0:
				obj.flagPosition = Top
			else:
				obj.flagPosition = Bottom
			
			obj.isFlashPoint = False	

		host.registerHandler('ControlPointChangedOwner', onCPStatusChangeFP)

		# setup ticket system
		ticketsTeam1 = calcStartTickets(bf2.gameLogic.getDefaultTickets(1))
		ticketsTeam2 = calcStartTickets(bf2.gameLogic.getDefaultTickets(2))
		
		bf2.gameLogic.setTickets(1, ticketsTeam1)
		bf2.gameLogic.setTickets(2, ticketsTeam2)

		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)

		bf2.gameLogic.setTicketLimit(1, 1, 0)
		bf2.gameLogic.setTicketLimit(2, 1, 0)
		bf2.gameLogic.setTicketLimit(1, 2, 10)
		bf2.gameLogic.setTicketLimit(2, 2, 10)
		bf2.gameLogic.setTicketLimit(1, 3, int(ticketsTeam1*0.1))
		bf2.gameLogic.setTicketLimit(2, 3, int(ticketsTeam2*0.1))
		bf2.gameLogic.setTicketLimit(1, 4, int(ticketsTeam1*0.2))
		bf2.gameLogic.setTicketLimit(2, 4, int(ticketsTeam1*0.2))
		
		host.registerHandler('TicketLimitReached', onTicketLimitReachedFP)

		# player events
		host.registerHandler('PlayerDeath', onPlayerDeathFP)
		host.registerHandler('PlayerKilled', onPlayerKilledFP)
		host.registerHandler('PlayerRevived', onPlayerRevivedFP)
		host.registerHandler('PlayerSpawn', onPlayerSpawnFP)
		host.registerHandler('EnterVehicle', onEnterVehicleFP)
		host.registerHandler('ExitVehicle', onExitVehicleFP)

		if g_debug: print "Flashpoint gamemode initialized."
	else:
		bf2.triggerManager.destroyAllTriggers()
		g_controlPoints = []


def initFlashPoint():
	FP.initialized = False
	FP.previousTeam = -1
	FP.nextPointTime = 5
	try:
		FP.timer.destroy()
		FP.timer = None
	except: pass
	
	try:
		FP.nextPointTimer.destroy()
		FP.nextPointTimer = None
	except: pass
	
def calcStartTickets(mapDefaultTickets):
	return int(mapDefaultTickets * (bf2.serverSettings.getTicketRatio() / 100.0))
	
	

def onTimeLimitReachedFP(value):
	team1tickets = bf2.gameLogic.getTickets(1)
	team2tickets = bf2.gameLogic.getTickets(2)
	
	winner = 0
	victoryType = 0
	if team1tickets > team2tickets:
		winner = 1
		victoryType = 3
	elif team2tickets > team1tickets:
		winner = 2
		victoryType = 3
	

	host.sgl_endGame(winner, victoryType, 0)

	
DOWNWARDS = -1
UPWARDS = 1	

# called when tickets reach a predetermined limit (negativ value means that the tickets have become less than the limit)
def onTicketLimitReachedFP(team, limitId):
	if (limitId == -1):
		if (team == 1):
			winner = 2
		
		elif (team == 2):
			winner = 1
		
		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)
	
		host.sgl_endGame(winner, 3, 0)		

	# update ticket state
	else:
		updateTicketWarning(team, limitId)

	

# called when the ticket state should be updated (for triggering messages and sounds based on tickets left)
def updateTicketWarning(team, limitId):

	oldTicketState = bf2.gameLogic.getTicketState(team)
	newTicketState = 0
	
	if (oldTicketState >= 20):
		newTicketState = 20		

	if (limitId == -2):
		newTicketState = 20
	
	elif (limitId == 2):
		newTicketState = 0		

	elif (limitId == -3):
		newTicketState += 1

	elif (limitId == -4):
		newTicketState += 2

	elif (limitId == -5):
		newTicketState += 3

	elif (limitId == -6):
		newTicketState += 4

	elif (limitId == -7):
		newTicketState += 5

	elif (limitId == -8):
		newTicketState += 6

	if (oldTicketState != newTicketState):
		bf2.gameLogic.setTicketState(team, newTicketState)
		
	
	
# called when someone enters or exits cp radius
def onCPTrigger(triggerId, cp, vehicle, enter, userData):
	if not cp.isValid(): return
	
	if not cp.isFlashPoint:
		if vehicle and vehicle.getParent():
			return
		
		playersInVehicle = None
		
		if vehicle:
			playersInVehicle = vehicle.getOccupyingPlayers()
			
		if enter:
			for p in playersInVehicle:
				cp = getOccupyingCP(p)
				if cp != None:
					if p.getIsInsideCP():
						p.setIsInsideCP(False)
		
		return
	
	if cp.isFlashPoint and vehicle == 'flashpoint':
		attackOverWeight = 5.0
		cp.cp_setParam('takeOverChangePerSecond', attackOverWeight)
		return
		

	if vehicle and vehicle.getParent(): return
	
	# can this cp be captured at all?
	if cp.cp_getParam('unableToChangeTeam') != 0:
		return

		
	playersInVehicle = None	
	if vehicle:
		playersInVehicle = vehicle.getOccupyingPlayers()
	
	if enter:
		for p in playersInVehicle:
			cp = getOccupyingCP(p)
			if cp != None:
				if not p.getIsInsideCP():
					if g_debug: print "Resetting enterPctAt for player ", p.getName()
					p.enterCpAt = host.timer_getWallTime()
	
	if vehicle:	
		for p in playersInVehicle:
			# only count first player in a vehicle
			if p == playersInVehicle[0]:  
				if g_debug: print p.index, " is in radius. v=", vehicle.templateName
				p.setIsInsideCP(enter)	
			else:
				p.setIsInsideCP(0)
				if enter == 1:
					bf2.gameLogic.sendHudEvent(p, 71, 49) #71 = HEEnableHelpMessage, 49 = VHMExitToCaptureFlag;


	# count people in radius
	team1Occupants = 0
	team2Occupants = 0

	pcos = bf2.triggerManager.getObjects(cp.triggerId)
	for o in pcos:
		if not o: continue # you can get None in the result tuple when the host can't figure out what object left the trigger
		if o.getParent(): continue # getOccupyingPlayers returns all players downwards in the hierarchy, so dont count them twice
		occupyingPlayers = o.getOccupyingPlayers()
		for p in occupyingPlayers:
			
			# only count first player in a vehicle
			if p != occupyingPlayers[0]: 
				continue
				
			if p.isAlive() and not p.isManDown():
				
				if not p.killed:
					if g_debug: print "Player ", p.getName(), " is inside radius"

					if p.getTeam() == 1:
						team1Occupants += 1
					elif p.getTeam() == 2:
						team2Occupants += 1


	# determine who is taking control
	team1OverWeight = team1Occupants - team2Occupants
	attackOverWeight = 0

	if team1OverWeight > 0:
		attackingTeam = 1
	elif team1OverWeight < 0:
		attackingTeam = 2
	else:
		attackingTeam = 0

	
	if team1Occupants == 0 and team2Occupants == 0:

		# nobody here, slowly go back to owning team
		if cp.cp_getParam('team') == 0:
			attackOverWeight = -0.5
		else:
			attackOverWeight = 0.5
			
		timeToChangeControl = cp.cp_getParam('timeToLoseControl')

	else:

		# raise flag if already ours, or at bottom and neutral. Otherwise lower first.
		if cp.cp_getParam('flag') == attackingTeam or (cp.flagPosition == Bottom and cp.cp_getParam('team') == 0):

			# our flag, raise
			attackOverWeight = abs(team1OverWeight)
			timeToChangeControl = cp.cp_getParam('timeToGetControl')
		else:

			# other team raised flag, lower first
			attackOverWeight = - abs(team1OverWeight)
			timeToChangeControl = cp.cp_getParam('timeToLoseControl')


	if cp.cp_getParam('onlyTakeableByTeam') != 0 and cp.cp_getParam('onlyTakeableByTeam') != attackingTeam:
		return


	# flag can only be changed when at bottom
	if cp.flagPosition == Bottom:
		cp.cp_setParam('flag', attackingTeam)


	# calculate flag raising/lowering speed
	if timeToChangeControl > 0:
		takeOverChangePerSecond = 1.0 * attackOverWeight / timeToChangeControl
	else:
		takeOverChangePerSecond = 0.0

	if (cp.flagPosition == Top and takeOverChangePerSecond > 0) or (cp.flagPosition == Bottom and takeOverChangePerSecond < 0):
		takeOverChangePerSecond = 0.0

	if abs(takeOverChangePerSecond) > 0:
		cp.flagPosition = Middle
				
	cp.cp_setParam('takeOverChangePerSecond', takeOverChangePerSecond)

			
	
# called when a control point flag reached top or bottom
def onCPStatusChangeFP(cp, top):

	playerId = -1
	takeoverType = -1
	newTeam = -1
	scoringTeam = -1
	
	if top:	cp.flagPosition = Top
	else:   cp.flagPosition = Bottom
	
	# determine capture / neutralize / defend
	if cp.cp_getParam('team') != 0:

		if top:
			# regained flag, do nothing
			pass
			
		else:
			# neutralize
			newTeam = 0
			if cp.cp_getParam('team') == 1:
				scoringTeam = 2
			else:
				scoringTeam = 1
				
			takeoverType = TAKEOVERTYPE_NEUTRALIZE

	else:

		if top:
			# capture
			newTeam = cp.cp_getParam('flag')
			scoringTeam = newTeam
			takeoverType = TAKEOVERTYPE_CAPTURE

		else:
			# hit bottom, but still neutral
			pass

	
	# scoring
	if takeoverType > 0:
		pcos = bf2.triggerManager.getObjects(cp.triggerId)
	
		# count number of players
		scoringPlayers = []
		firstPlayers = []
		for o in pcos:
			if o.getParent(): continue

			occupyingPlayers = o.getOccupyingPlayers()
			for p in occupyingPlayers:
			
				# only count first player in a vehicle
				if p != occupyingPlayers[0]: 
					continue
					
				if p.isAlive() and not p.isManDown() and p.getTeam() == scoringTeam:
					if len(firstPlayers) == 0 or p.enterCpAt < firstPlayers[0].enterCpAt:
						firstPlayers = [p]
					elif p.enterCpAt == firstPlayers[0].enterCpAt:
						firstPlayers += [p]
					
					if not p in scoringPlayers:
						scoringPlayers += [p]
	
		# deal score
		for p in scoringPlayers:
			oldScore = p.score.score;
			if takeoverType == TAKEOVERTYPE_CAPTURE:
				if p in firstPlayers:
					p.score.cpCaptures += 1
					addScore(p, SCORE_CAPTURE, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 0) #12 = Conquest, 0 = Capture
					playerId = p.index
				else:
					p.score.cpAssists += 1
					addScore(p, SCORE_CAPTUREASSIST, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 2) #12 = Conquest, 2 = Assist

			elif takeoverType == TAKEOVERTYPE_NEUTRALIZE:
				if p in firstPlayers:
					p.score.cpNeutralizes += 1
					addScore(p, SCORE_NEUTRALIZE, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 3) #12 = Conquest, 3 = Neutralize
					playerId = p.index
				else:
					p.score.cpNeutralizeAssists += 1
					addScore(p, SCORE_NEUTRALIZEASSIST, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 4) #12 = Conquest, 4 = Neutralize assist
	
	
	# update control point
	cp.cp_setParam('playerId', playerId) #always set player first
	if newTeam != -1 and cp.cp_getParam('team') != newTeam:
		cp.cp_setParam('team', newTeam)
	onCPTrigger(cp.triggerId, cp, 0, 0, 0)
	
	if cp.isFlashPoint and takeoverType == TAKEOVERTYPE_CAPTURE:
		# ===================== flashpoint additions ========================#
		cp.cp_setParam('team', 0)
		cp.cp_setParam('flag', 0)
		cp.isFlashPoint = False
		removePlayers()
		onCPTrigger(cp.triggerId, cp, 0, 0, 0)
		
		#instant ticket loss when control is captured
		ticketPenalty = 10
		penalizedTeam = -1
		if newTeam == 1:
			penalizedTeam = 2
			ticketsPenalizedTeam = bf2.gameLogic.getTickets(penalizedTeam)
			ticketsPenalizedTeam -= ticketPenalty
			bf2.gameLogic.setTickets(penalizedTeam, ticketsPenalizedTeam)
			
		elif newTeam == 2:
			penalizedTeam = 1
			ticketsPenalizedTeam = bf2.gameLogic.getTickets(penalizedTeam)
			ticketsPenalizedTeam -= ticketPenalty
			bf2.gameLogic.setTickets(penalizedTeam, ticketsPenalizedTeam)
			
		FP.timer.destroy()
		FP.timer = None
			
		#start next flashpoint
		FP.nextPointTimer = bf2.Timer(nextFlashPoint, 0, 1)
		FP.nextPointTimer.setRecurring(1.0)

				
def onPlayerDeathFP(victim, vehicle):		
	if g_debug: print 'reserved for future use'	

def onPlayerKilledFP(victim, attacker, weapon, assists, object):
	if not victim: 
		return
	
	victim.killed = True
	
	# update flag takeover status if victim was in a CP radius
	cp = getOccupyingCP(victim)
	if cp != None:
		onCPTrigger(-1, cp, victim.getVehicle(), False, None)

		# give defend score if killing enemy within cp radius
		if attacker != None and attacker.getTeam() != victim.getTeam()\
		   and cp.cp_getParam('unableToChangeTeam') == 0 and cp.cp_getParam('onlyTakeableByTeam') == 0:
		
			if cp != None and cp.cp_getParam('team') == attacker.getTeam():
				attacker.score.cpDefends += 1
				addScore(attacker, SCORE_DEFEND, RPL)
				bf2.gameLogic.sendGameEvent(attacker, 12, 1) #12 = Conquest, 1 = Defend
			
			
			
def onPlayerRevivedFP(victim, attacker):

	# update flag takeover status if victim was in a CP radius
	victim.killed = False
	
	cp = getOccupyingCP(victim)
	if cp != None:
		onCPTrigger(-1, cp, victim.getVehicle(), True, None)
	
	
	
def onPlayerSpawnFP(player, soldier):
	player.killed = False
	
	if not FP.initialized:
		FP.initialized = True
		#start next flashpoint
		FP.nextPointTimer = bf2.Timer(nextFlashPoint, 10, 1)
		FP.nextPointTimer.setRecurring(1.0)
	
def onEnterVehicleFP(player, vehicle, freeSoldier = False):
	rootVehicle = bf2.objectManager.getRootParent(vehicle)
	# update flag takeover status if player in a CP radius
	if player == None:
		return
	cp = getOccupyingCP(player)
	playersInVehicle = rootVehicle.getOccupyingPlayers()
	if cp != None and player == playersInVehicle[0]:
		occupyingPlayers = rootVehicle.getOccupyingPlayers()
		for p in occupyingPlayers:
			p.setIsInsideCP(False)
		if cp.cp_getParam('unableToChangeTeam') == 0:
			player.setIsInsideCP(True)
	else:		
		player.setIsInsideCP(False)



def onExitVehicleFP(player, vehicle):
	# Player remaining in vehicle might capture CP
	playersInVehicle = vehicle.getOccupyingPlayers()
	if len(playersInVehicle) != 0:
		remainingPlayer = playersInVehicle[0]
		if remainingPlayer != None:
			cpRm = getOccupyingCP(remainingPlayer)
			if cpRm != None and cpRm.cp_getParam('unableToChangeTeam') == 0:
				remainingPlayer.setIsInsideCP(True)

	# update flag takeover status if player in a CP radius
	cp = getOccupyingCP(player)
	if cp != None and cp.cp_getParam('unableToChangeTeam') == 0:
		player.setIsInsideCP(True)
	else:		
		player.setIsInsideCP(False)
		
		
		
# find cp that player is occupying, if any		
def getOccupyingCP(player):	
	if player == None: return

	vehicle = player.getVehicle()
	playerPos = vehicle.getPosition()
	
	# find closest CP
	closestCP = None
	if len(g_controlPoints) == 0: return None
	for obj in g_controlPoints:
		distanceTo = getVectorDistance(playerPos, obj.getPosition())
		if closestCP == None or distanceTo < closestCPdist:
			closestCP = obj
			closestCPdist = distanceTo
	
	# is the player in radius?
	pcos = bf2.triggerManager.getObjects(closestCP.triggerId)
	for o in pcos:
		if o == player.getDefaultVehicle():
			# Player is DEFAULT vehicle - this is needed when called from onEnterVehicle
			return closestCP
		else:
			for p in o.getOccupyingPlayers():
				if p == player:
					return closestCP
	
	return None



# get distance between two positions
def getVectorDistance(pos1, pos2):
	diffVec = [0.0, 0.0, 0.0]
	diffVec[0] = math.fabs(pos1[0] - pos2[0])
	diffVec[1] = math.fabs(pos1[1] - pos2[1])
	diffVec[2] = math.fabs(pos1[2] - pos2[2])
	 
	return math.sqrt(diffVec[0] * diffVec[0] + diffVec[1] * diffVec[1] + diffVec[2] * diffVec[2])
		
	
def startFlashPoint():
	numFlashPoints = len(g_controlPoints)
	
	cpNum = random.randint(0, (numFlashPoints -1))
	cp = g_controlPoints[cpNum]
	while cp.cp_getParam('unableToChangeTeam'):
		cpNum = random.randint(0, (numFlashPoints -1))
		cp = g_controlPoints[cpNum]
	
	cp = g_controlPoints[cpNum]
	
	if FP.currentTeam == -1:
		team = random.randint(1, 2)
		FP.currentTeam = team
		if team == 1:
			FP.previousTeam = 2
			
		elif team == 2:
			FP.previousTeam = 1

	
	elif FP.currentTeam == 1:
		FP.currentTeam = 2
		FP.previousTeam = 1
		
	elif FP.currentTeam == 2:
		FP.currentTeam = 1
		FP.previousTeam = 2
		
	cp.cp_setParam('team', FP.currentTeam)
	cp.cp_setParam('flag', FP.currentTeam)
	cp.isFlashPoint = True
	onCPTrigger(-1, cp, "flashpoint", True, None)
	host.rcon_invoke("game.sayAll \"" + " |C1001 Flashpoint Active! Secure the sector soldier! |C1001 "  + "\"")
	host.rcon_invoke("game.sayAll \"" + " |C1001 Flashpoint Active! Secure the sector soldier! |C1001 "  + "\"")
	host.rcon_invoke("game.sayAll \"" + " |C1001 Flashpoint Active! Secure the sector soldier! |C1001 "  + "\"")
	FP.timer = bf2.Timer(flashPoint, 180, 1, cp)
	
	
def flashPoint(obj):
	if FP.currentTeam != FP.previousTeam:
		if FP.currentTeam == 1:
			host.rcon_invoke("game.sayAll \"" + " |C1001 Flashpoint won by PAC |C1001 "  + "\"")
			host.rcon_invoke("game.sayAll \"" + " |C1001 Flashpoint won by PAC |C1001 "  + "\"")
			host.rcon_invoke("game.sayAll \"" + " |C1001 Flashpoint won by PAC |C1001 "  + "\"")
		
		elif FP.currentTeam == 2:
			host.rcon_invoke("game.sayAll \"" + " |C1001 Flashpoint won by EU |C1001 "  + "\"")
			host.rcon_invoke("game.sayAll \"" + " |C1001 Flashpoint won by EU |C1001 "  + "\"")
			host.rcon_invoke("game.sayAll \"" + " |C1001 Flashpoint won by EU |C1001 "  + "\"")
		
		#instant ticket loss for expired timer
		ticketPenalty = 10
		penalizedTeam = -1
		
		if FP.currentTeam == 1:
			penalizedTeam = 2
			ticketsPenalizedTeam = bf2.gameLogic.getTickets(penalizedTeam)
			ticketsPenalizedTeam -= ticketPenalty
			bf2.gameLogic.setTickets(penalizedTeam, ticketsPenalizedTeam)
			
		elif FP.currentTeam == 2:
			penalizedTeam = 1
			ticketsPenalizedTeam = bf2.gameLogic.getTickets(penalizedTeam)
			ticketsPenalizedTeam -= ticketPenalty
			bf2.gameLogic.setTickets(penalizedTeam, ticketsPenalizedTeam)
			
		#start next flashpoint
		FP.nextPointTimer = bf2.Timer(nextFlashPoint, 10, 1)
		FP.nextPointTimer.setRecurring(1.0)
	
	obj.cp_setParam('team', 0)
	obj.cp_setParam('flag', 0)
	obj.isFlashPoint = False
	removePlayers()
		
	FP.timer.destroy()
	FP.timer = None

def removePlayers():
	for p in bf2.playerManager.getPlayers():
		cp = getOccupyingCP(p)
		if cp != None:
			onCPTrigger(-1, cp, p.getVehicle(), True, None)
			
def nextFlashPoint(data):		
	host.rcon_invoke("game.sayAll \"" + " |C1001 Attention Soldier next Flashpoint activating.. |C1001 "  + "\"")
	FP.nextPointTime -= 1
	if FP.nextPointTime <= 0:
		FP.nextPointTime = 10
		startFlashPoint()
		FP.nextPointTimer.destroy()
		FP.nextPointTimer = None
